# gae
Graphs Ain't Easy language

## Progress checklist for the rest of the language
- [x] semant.ml
  - [x] expr
    - [x] Map
    - [x] Graph
    - [x] Edge
    - [x] Binop
- [ ] LLVM
  - [ ] expr builder
    - [x] built-in types (e.g. double, int, etc)
    - [ ] data structures
      - [x] string
        - [x] built-in functions (implemented in C and used in `expr`)
          - [x] init (not exposed)
          - [x] length (e.g. `len(s)`)
          - [x] comparison (e.g. `s1 == s2`, `s1 < s2`)
          - [x] concat (e.g. s1 + s2)
      - [ ] array
        - [x] built-in functions (implemented in C and used in `expr`)
          - [x] init
            - [x] expose `array_init()`
          - [x] get (e.g. `a[i]`)
          - [x] set (e.g. `a[i] = 5`)
          - [x] length (e.g. `lena(a)`)
        - [ ] string_of_array
      - [ ] map
        - [x] built-in functions (implemented in C and used in `expr`)
          - [x] init
            - [x] expose `map_init()`
          - [x] get (e.g. `m[k]`)
          - [x] set (e.g. `m[k] = v`)
          - [x] length (e.g. `lenm(m)`)
          - [x] getKeys (e.g. `getKeys(m)`) so we can iterate through it
        - [ ] string_of_map (lo-pri)
        - [x] change value to be `void *` type so it support any arbitrary data trpe
      - [ ] struct (dependent on string)
        - [x] built-in functions (implemented in C and used in `expr`)
          - [x] init (not exposed)
          - [x] get (e.g. `s.field`)
          - [x] set (e.g. `s.field = 5`)
          - [x] equality (e.g. `s == t`)
      - [ ] edge (dependent on struct)
        - [x] built-in functions (implemented in C and used in `expr`)
          - [x] init (not exposed)
          - [x] getSrcNode (`getSrc(e)` => node struct)
          - [x] getDstNode (`getDst(e)` => node struct)
          - [x] getValue (`getVal(e)` => edge value struct)
          - [x] setSrcNode (`setSrc(e, node_struct)`)
          - [x] setDstNode (`setDst(e, node_struct)`)
          - [x] setValue (`setVal(e, value_struct)`)
      - [ ] graph (dependent on edge)
        - [x] prereqs
          - [x] map support for backend NodePair struct
        - [ ] built-in functions (implemented in C and used in `expr`)
          - [x] init
            - [x] expose `graph_init()`
          - [x] getNodes (`getNodes(g)` => list of node structs)
          - [x] getEdges (`getEdges(g, n)` => list of edges with node n as src)
          - [ ] updateNode (`updateNode(g, old_node, new_node)`)
          - [ ] updateEdge (`updateEdge(g, old_edge, new_edge)`)
          - [x] addEdge (`addEdge(g, new_edge)`)
          - [ ] removeEdge (`removeEdge(g, edge)`)
          - [ ] removeNode (`removeNode(g, node)` => calls `removeEdge` for all edges from/to `node`)
    - [ ] binops
      - [x] int/double ops
      - [ ] in
        - [x] array
        - [x] map
        - [ ] graph
          - [x] struct equality operator
          - [x] node in graph
          - [ ] edge in graph
    - [x] unops
      - [x] ++
      - [x] --
    - [x] other built-in functions
      - [x] print*
      - [x] int_of_double/double_of_int
  - [ ] stmt builder
    - [x] if
      - note: elseifs aren't implemented, workaround is to tell programmer to use nested if blocks
    - [x] for/while
    - [ ] continue/break
- [ ] Cleaning up/Misc
  - [x] Fixing warnings in `semant.ml`
  - [ ] Fixing warnings in `codegen.ml`
  - [ ] Writing tests for `semant.ml`
  - [ ] Pretty-print functions

## Progress checklist for Hello World
- [x] AST
  - [x] Get `parser.native` to compile properly
  - [x] Write tests to verify AST output from Menhir
- [x] Semantic AST
  - [x] sast.ml
  - [x] semant.ml
    - [x] checks for function, struct, bind
    - [x] check_expr (ish)
    - [x] check_stmt (ish)
- [x] Misc
    - [x] Create Makefile to automate build process
    - [x] Address existing TODOs in `parser.mly`

## Progress checklist for Scanner + Parser
- [x] Scanner
- [x] Parser
  - [x] typ
  - [x] fdecl
  - [x] stmt
    - [x] add if elseif else, and other control flow rules
  - [x] expr
    - [x] container data type literals
      - [x] struct
      - [x] array
      - [x] map
      - [x] graph
