(* Ocamllex scanner for GaE *)

{ open Parser }

let Exp = ['e' 'E']['+' '-']?['0'-'9']+

rule token = parse
  [' ' '\t' '\r' '\n'] { token lexbuf } (* Whitespace *)
| "/*"     { comment lexbuf }           (* Comments *)
| '('      { LPAREN }
| ')'      { RPAREN }
| '['      { LBRACKET }
| ']'      { RBRACKET }
| '{'      { LBRACE }
| '}'      { RBRACE }
| ':'      { COLON }
| ';'      { SEMI }
| ','      { COMMA }
| '.'      { PERIOD }
| '+'      { PLUS }
| '-'      { MINUS }
| '*'      { TIMES }
| '/'      { DIVIDE }
| '%'      { MOD }
| "++"     { INCR }
| "--"     { DECR }
| "+."     { DPLUS }
| "-."     { DMINUS }
| "*."     { DTIMES }
| "/."     { DDIVIDE }
| "%."     { DMOD }
| '='      { REASSIGN }
| ":="     { ASSIGN }
| "=="     { EQ }
| "!="     { NEQ }
| '<'      { LT }
| "<="     { LEQ }
| ">"      { GT }
| ">="     { GEQ }
| "&&"     { AND }
| "||"     { OR }
| "!"      { NOT }
| "if"     { IF }
| "elseif" { ELSEIF }
| "else"   { ELSE }
| "for"    { FOR }
| "while"  { WHILE }
| "return" { RETURN }
| "continue" { CONTINUE }
| "in"     { IN }
| "func"   { FUNC }
| "struct" { STRUCT }
| "int"    { INT }
| "double" { DOUBLE }
| "bool"   { BOOL }
| "char"   { CHAR }
| "string" { STRING }
| "graph"  { GRAPH }
| "edge"   { EDGE }
| "map"    { MAP }
| "break"  { BREAK }
| "true"   { TRUE }
| "false"  { FALSE }
| ['0'-'9']+ as lxm { INTLIT(int_of_string lxm) }
| '\'' [ ^'\'' ]  '\'' as charlit  { CHARLIT(String.sub charlit 1 ((String.length charlit) - 2)) }
| '"' [ ^'"'  ]* '"'   as strlit { STRLIT(String.sub strlit 1 ((String.length strlit) - 2)) }
| ( '.' ['0'-'9']+ Exp? | ['0'-'9']+ ( '.' ['0'-'9']* Exp? | Exp)) as lxm { DUBLIT(float_of_string lxm) }
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']*[':'] as lxm { FIELD(String.sub lxm 0 ((String.length lxm) - 1)) }
| ['a'-'z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| ['A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { STRUCTID(lxm) }
| eof { EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }

and comment = parse
  "*/" { token lexbuf }
| _    { comment lexbuf }
